@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.930'; # TRIAL
+$Email::MIME::VERSION = '1.931'; # TRIAL
 use Email::Simple 2.104; # header_raw_*
 use parent qw(Email::Simple);
 
@@ -72,7 +72,7 @@ use Scalar::Util qw(reftype);
 #pod         Email::MIME->create( parts => [ @parts ] ),
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   # standard modifications
 #pod   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
 #pod   $email->header_str_set( To            => rcpts()        );
@@ -81,7 +81,7 @@ use Scalar::Util qw(reftype);
 #pod
 #pod   # more advanced
 #pod   $_->encoding_set( 'base64' ) for $email->parts;
-#pod   
+#pod
 #pod   # Quick multipart creation
 #pod   my $quicky = Email::MIME->create(
 #pod       header_str => [
@@ -94,9 +94,9 @@ use Scalar::Util qw(reftype);
 #pod           q[These could be binary too],
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   print $email->as_string;
-#pod   
+#pod
 #pod =head1 DESCRIPTION
 #pod
 #pod This is an extension of the L<Email::Simple> module, to handle MIME
@@ -433,6 +433,11 @@ sub invent_filename {
 
 sub default_header_class { 'Email::MIME::Header' }
 
+sub header_str {
+  my $self = shift;
+  $self->header_obj->header_str(@_);
+}
+
 sub header_str_set {
   my $self = shift;
   $self->header_obj->header_str_set(@_);
@@ -703,9 +708,12 @@ sub parts_set {
     delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
     $body .= $parts->[0]->body;
-    @{$ct_header}{qw[type subtype]}
-      = @{ parse_content_type($parts->[0]->header('Content-Type')) }
-      {qw[type subtype]};
+
+    my $from_ct = parse_content_type($parts->[0]->header('Content-Type'));
+    @{$ct_header}{qw[type subtype]} = @{ $from_ct }{qw[type subtype]};
+
+    $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+
     $self->encoding_set($parts->[0]->header('Content-Transfer-Encoding'));
     delete $ct_header->{attributes}->{boundary};
   }
@@ -736,7 +744,7 @@ sub parts_add {
 #pod   $email->walk_parts(sub {
 #pod       my ($part) = @_;
 #pod       return if $part->subparts; # multipart
-#pod       
+#pod
 #pod       if ( $part->content_type =~ m[text/html]i ) {
 #pod           my $body = $part->body;
 #pod           $body =~ s/<link [^>]+>//; # simple filter example
@@ -848,7 +856,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.930
+version 1.931
 
 =head1 SYNOPSIS
 
@@ -904,7 +912,7 @@ by all means keep reading.
         Email::MIME->create( parts => [ @parts ] ),
       ],
   );
-  
+
   # standard modifications
   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
   $email->header_str_set( To            => rcpts()        );
@@ -913,7 +921,7 @@ by all means keep reading.
 
   # more advanced
   $_->encoding_set( 'base64' ) for $email->parts;
-  
+
   # Quick multipart creation
   my $quicky = Email::MIME->create(
       header_str => [
@@ -926,7 +934,7 @@ by all means keep reading.
           q[These could be binary too],
       ],
   );
-  
+
   print $email->as_string;
 
 =head1 DESCRIPTION
@@ -1076,7 +1084,7 @@ of additional parts.
   $email->walk_parts(sub {
       my ($part) = @_;
       return if $part->subparts; # multipart
-      
+
       if ( $part->content_type =~ m[text/html]i ) {
           my $body = $part->body;
           $body =~ s/<link [^>]+>//; # simple filter example
